"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var DefaultLibraryLoader_exports = {};
__export(DefaultLibraryLoader_exports, {
  DefaultLibraryLoader: () => DefaultLibraryLoader,
  load: () => load
});
module.exports = __toCommonJS(DefaultLibraryLoader_exports);
var import_debug = __toESM(require("@prisma/debug"));
var import_engines = require("@prisma/engines");
var import_get_platform = require("@prisma/get-platform");
var import_fs = __toESM(require("fs"));
var import_colors = require("kleur/colors");
var import_os = __toESM(require("os"));
var import_path = __toESM(require("path"));
var import_PrismaClientInitializationError = require("../common/errors/PrismaClientInitializationError");
var import_handleEngineLoadingErrors = require("../common/errors/utils/handleEngineLoadingErrors");
var import_printGeneratorConfig = require("../common/utils/printGeneratorConfig");
var import_util = require("../common/utils/util");
var import_tracing = require("../tracing");
const debug = (0, import_debug.default)("prisma:client:libraryEngine:loader");
const libraryCacheSymbol = Symbol("PrismaLibraryEngineCache");
function getLibraryCache() {
  const globalWithCache = globalThis;
  if (globalWithCache[libraryCacheSymbol] === void 0) {
    globalWithCache[libraryCacheSymbol] = {};
  }
  return globalWithCache[libraryCacheSymbol];
}
__name(getLibraryCache, "getLibraryCache");
function load(libraryPath) {
  const cache = getLibraryCache();
  if (cache[libraryPath] !== void 0) {
    return cache[libraryPath];
  }
  const fullLibraryPath = import_path.default.toNamespacedPath(libraryPath);
  const libraryModule = { exports: {} };
  let flags = 0;
  if (process.platform !== "win32") {
    flags = import_os.default.constants.dlopen.RTLD_LAZY | import_os.default.constants.dlopen.RTLD_DEEPBIND;
  }
  process.dlopen(libraryModule, fullLibraryPath, flags);
  cache[libraryPath] = libraryModule.exports;
  return libraryModule.exports;
}
__name(load, "load");
class DefaultLibraryLoader {
  constructor(config) {
    this.libQueryEnginePath = null;
    this.platform = null;
    this.config = config;
  }
  async loadLibrary() {
    const platformInfo = await (0, import_get_platform.getPlatformWithOSResult)();
    this.platform = platformInfo.binaryTarget;
    if (!this.libQueryEnginePath) {
      this.libQueryEnginePath = await this.getLibQueryEnginePath();
    }
    debug(`loadEngine using ${this.libQueryEnginePath}`);
    try {
      const enginePath2 = this.libQueryEnginePath;
      return (0, import_tracing.runInChildSpan)(
        { name: "loadLibrary", enabled: this.config.tracingConfig.enabled, internal: true },
        () => load(enginePath2)
      );
    } catch (e) {
      const errorMessage = (0, import_handleEngineLoadingErrors.handleLibraryLoadingErrors)({
        e,
        platformInfo,
        id: this.libQueryEnginePath
      });
      throw new import_PrismaClientInitializationError.PrismaClientInitializationError(errorMessage, this.config.clientVersion);
    }
  }
  async getLibQueryEnginePath() {
    const libPath = process.env.PRISMA_QUERY_ENGINE_LIBRARY ?? this.config.prismaPath;
    if (libPath && import_fs.default.existsSync(libPath) && libPath.endsWith(".node")) {
      return libPath;
    }
    this.platform = this.platform ?? await (0, import_get_platform.getPlatform)();
    const { enginePath: enginePath2, searchedLocations: searchedLocations2 } = await this.resolveEnginePath();
    if (!import_fs.default.existsSync(enginePath2)) {
      const incorrectPinnedPlatformErrorStr = this.platform ? `
You incorrectly pinned it to ${(0, import_colors.bold)((0, import_colors.red)(`${this.platform}`))}
` : "";
      let errorText = `Query engine library for current platform "${(0, import_colors.bold)(
        this.platform
      )}" could not be found.${incorrectPinnedPlatformErrorStr}
This probably happens, because you built Prisma Client on a different platform.
(Prisma Client looked in "${(0, import_colors.underline)(enginePath2)}")

Searched Locations:

${searchedLocations2.map((f) => {
        let msg = `  ${f}`;
        if (process.env.DEBUG === "node-engine-search-locations" && import_fs.default.existsSync(f)) {
          const dir = import_fs.default.readdirSync(f);
          msg += dir.map((d) => `    ${d}`).join("\n");
        }
        return msg;
      }).join("\n" + (process.env.DEBUG === "node-engine-search-locations" ? "\n" : ""))}
`;
      if (this.config.generator) {
        this.platform = this.platform ?? await (0, import_get_platform.getPlatform)();
        if (this.config.generator.binaryTargets.find((object) => object.value === this.platform) || this.config.generator.binaryTargets.find((object) => object.value === "native")) {
          errorText += `
You already added the platform${this.config.generator.binaryTargets.length > 1 ? "s" : ""} ${this.config.generator.binaryTargets.map((t) => `"${(0, import_colors.bold)(t.value)}"`).join(", ")} to the "${(0, import_colors.underline)(
            "generator"
          )}" block
in the "schema.prisma" file as described in https://pris.ly/d/client-generator,
but something went wrong. That's suboptimal.

Please create an issue at https://github.com/prisma/prisma/issues/new`;
          errorText += ``;
        } else {
          errorText += `

To solve this problem, add the platform "${this.platform}" to the "${(0, import_colors.underline)(
            "binaryTargets"
          )}" attribute in the "${(0, import_colors.underline)("generator")}" block in the "schema.prisma" file:
${(0, import_colors.green)(this.getFixedGenerator())}

Then run "${(0, import_colors.green)("prisma generate")}" for your changes to take effect.
Read more about deploying Prisma Client: https://pris.ly/d/client-generator`;
        }
      } else {
        errorText += `

Read more about deploying Prisma Client: https://pris.ly/d/client-generator
`;
      }
      throw new import_PrismaClientInitializationError.PrismaClientInitializationError(errorText, this.config.clientVersion);
    }
    this.platform = this.platform ?? await (0, import_get_platform.getPlatform)();
    return enginePath2;
  }
  async resolveEnginePath() {
    const searchedLocations = [];
    let enginePath;
    if (this.libQueryEnginePath) {
      return { enginePath: this.libQueryEnginePath, searchedLocations };
    }
    this.platform = this.platform ?? await (0, import_get_platform.getPlatform)();
    if (__filename.includes("DefaultLibraryLoader")) {
      enginePath = import_path.default.join((0, import_engines.getEnginesPath)(), (0, import_get_platform.getNodeAPIName)(this.platform, "fs"));
      return { enginePath, searchedLocations };
    }
    const dirname = eval("__dirname");
    const searchLocations = [
      import_path.default.resolve(dirname, "../../../.prisma/client"),
      this.config.generator?.output?.value ?? dirname,
      import_path.default.resolve(dirname, ".."),
      import_path.default.dirname(this.config.datamodelPath),
      this.config.cwd,
      "/tmp/prisma-engines"
    ];
    if (this.config.dirname) {
      searchLocations.push(this.config.dirname);
    }
    for (const location of searchLocations) {
      searchedLocations.push(location);
      debug(`Searching for Query Engine Library in ${location}`);
      enginePath = import_path.default.join(location, (0, import_get_platform.getNodeAPIName)(this.platform, "fs"));
      if (import_fs.default.existsSync(enginePath)) {
        return { enginePath, searchedLocations };
      }
    }
    enginePath = import_path.default.join(__dirname, (0, import_get_platform.getNodeAPIName)(this.platform, "fs"));
    return { enginePath, searchedLocations };
  }
  getFixedGenerator() {
    const fixedGenerator = {
      ...this.config.generator,
      binaryTargets: (0, import_util.fixBinaryTargets)(this.config.generator.binaryTargets, this.platform)
    };
    return (0, import_printGeneratorConfig.printGeneratorConfig)(fixedGenerator);
  }
}
__name(DefaultLibraryLoader, "DefaultLibraryLoader");
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  DefaultLibraryLoader,
  load
});
